
/*********************************************************************************

Copyright(c) 2014 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      watchdog.c
*
* @brief     Primary source file for Watchdog Timer example.
*
* @details
 *          This test program checks the functionality of the watch dog timer by
 *
 *          1. Checking the counter and watch dog status immediately after enabling it.
 *             They should not have expired at this point.
 *          2. Reseting the watch dog timer during a Delay() function. The counter and
 *             watch dog should not expire after the Delay() has completed.
 *          3. Not reseting the watch dog timer during a Delay() function. The counter and
 *             watch dog should expire after the Delay() has completed.
*
*/

#include <stdio.h>
#include <adi_types.h>
#include <services/wd/adi_wd.h>
/* Managed drivers and/or services include */
#if  defined(__ADSPSC589_FAMILY__)
	#include "adi_initialize.h"
#else
	#include "system/adi_initialize.h"
#endif


extern void ConfigSoftSwitches(void);

#define WATCHDOG 0x00800000u
uint32_t WatchDogTest(void);
void Delay(bool bResetWatchdog);
void WatchDog_Callback(void *pCBParam, uint32_t nEvent, void *pArg);

static ADI_WD_HANDLE phWD0 = NULL;
bool bWatchDogReset = false;
/* callback flag */
volatile bool bCallbackFlag = false;

int main(void)
{
    uint32_t Result = 0u;

    /* Initialize managed drivers and/or services */
    adi_initComponents();

    ConfigSoftSwitches();

    /* General purpose interrupt watchdog test */
    if (Result == 0u)
    {
       Result = (uint32_t)adi_wd_Open(0u, &phWD0, WatchDog_Callback, NULL);
    }





    /* do test */
    if (Result == 0u)
    {
       Result = WatchDogTest();
    }
    adi_wd_Enable(phWD0, false);
    adi_wd_Close(phWD0);

    /* Output results */
    if (Result == 0u)
    {
        printf("All done\n");
    }
    else
    {
        printf("Example failed\n");
    }
    return 0;
}

uint32_t WatchDogTest(void)
{
    uint32_t Result = 0u;
    bool bRollOver;
    uint32_t Counter;
    /* set watch dog timer counter */
    if (Result == 0u)
    {
       Result = (uint32_t)adi_wd_SetCounter(phWD0, WATCHDOG);
    }

    /* enable watch dog */
    if (Result == 0u)
    {
        Result = (uint32_t)adi_wd_Enable(phWD0, true);
    }
    /* Test1: Check watch dog Counter and bRollOver have not expired, at this point */
    /* get watch dog timer */
    if (Result == 0u)
    {
        Result = (uint32_t)adi_wd_GetCounter(phWD0, &Counter);
    }
    /* check watch dog status */
    if (Result == 0u)
    {
        Result = (uint32_t)adi_wd_GetRollOverStatus(phWD0, &bRollOver);
    }
    /* check watch dog status and counter */
    if (Result == 0u)
    {
        if((bRollOver == false) && (Counter !=0u))
        {
            Result = 0u;
        }
        else
        {
            Result = 1u;
        }
    }

    /* Test 2: Reset watch dog during Delay(), Check watch dog Counter and bRollOver have not expired, at this point */
    Delay(true);
    /* get watch dog timer */
    if (Result == 0u)
    {
        Result = (uint32_t)adi_wd_GetCounter(phWD0, &Counter);
    }
    /* check watch dog status */
    if (Result == 0u)
    {
        Result = (uint32_t)adi_wd_GetRollOverStatus(phWD0, &bRollOver);
    }

    /* check watch dog status and counter */
    if (Result == 0u)
    {
        if((bRollOver == false) && (Counter !=0u))
        {
            Result = 0u;
        }
    else
    {
            Result = 1u;
        }
    }
    /* Test 3: Don't reset watch dog during Delay(), should get a watchdog expired event */
    Delay(false);
    if(bCallbackFlag == false)
    {
        /* Test failed */
        Result = 1u;
    }

    return Result;
}

/* small delay  */
void Delay(bool bResetWatchdog)
{
    volatile uint32_t i=0u;

    while(i < WATCHDOG)
    {
        i++;
        if(bResetWatchdog == true)
        {
            adi_wd_Reset(phWD0, WATCHDOG);
        }
    }

}

void WatchDog_Callback(void *pCBParam, uint32_t nEvent, void *pArg)
{
    switch(nEvent)
    {
        case (uint32_t)ADI_WD_EVENT_GP_INT:
            bCallbackFlag = true;
            adi_wd_Enable(phWD0, false);
            adi_wd_Reset(phWD0, WATCHDOG);
            break;
        default:
            break;
    }
}

